from .rcnet import RCNetCentered_3x3, RCNetCentered_7x7, RCNetRot90_3x3, RCNetRot90_7x7, RCNetx1, RCNetx2
from .rclut import RCLutCentered_3x3, RCLutCentered_7x7, RCLutRot90_3x3, RCLutRot90_7x7, RCLutx1, RCLutx2
from .srnet import SRNet, SRNetRot90
from .srlut import SRLut, SRLutRot90
import torch
import numpy as np 
from pathlib import Path

AVAILABLE_MODELS = {
    'SRNet': SRNet, 'SRLut': SRLut,
    'SRNetRot90': SRNetRot90, 'SRLutRot90': SRLutRot90,
    'RCNetCentered_3x3': RCNetCentered_3x3, 'RCLutCentered_3x3': RCLutCentered_3x3,   
    'RCNetCentered_7x7': RCNetCentered_7x7, 'RCLutCentered_7x7': RCLutCentered_7x7,   
    'RCNetRot90_3x3': RCNetRot90_3x3, 'RCLutRot90_3x3': RCLutRot90_3x3,
    'RCNetRot90_7x7': RCNetRot90_7x7, 'RCLutRot90_7x7': RCLutRot90_7x7,
    'RCNetx1': RCNetx1, 'RCLutx1': RCLutx1,
    'RCNetx2': RCNetx2, 'RCLutx2': RCLutx2,
}

def SaveCheckpoint(model, path):
    model_container = {
        'model': model.__class__.__name__,
        'state_dict': model.state_dict(),
        **{k:v for k,v in model.__dict__.items() if k[0] != '_' and k != 'training'}
    }
    torch.save(model_container, path)

def LoadCheckpoint(model_path):
    model_path = Path(model_path).absolute()
    if model_path.exists():
        model_container = torch.load(model_path)
        model = AVAILABLE_MODELS[model_container['model']](**{k:v for k,v in model_container.items() if k != "model" and k != "state_dict"})
        model.load_state_dict(model_container['state_dict'], strict=True)
        return model
    else:
        raise Exception(f"Path {model_path} does not exist.")